import graph.*;
import javax.swing.JTextPane;
public class MSBKruskal {
  private Graph bspGraph; 
  private String markierteKanten="";
  private JTextPane textfeld;

  public MSBKruskal(Graph graph, JTextPane pText) {
    bspGraph = graph;
   textfeld = pText;
  }

  private List<Edge> gibAufsteigendSortiert(List<Edge> pList){
	    List<Edge> ergebnis = new List<Edge>();

	    //Alle Kanten von pList durchlaufen.
	    pList.toFirst();
	    while (pList.hasAccess()){
	      Edge aktuell = pList.getContent();

	      //Kante sortiert in die Ergebnisliste einfuegen.
	      ergebnis.toFirst();
	      while (ergebnis.hasAccess() &&  ergebnis.getContent().getWeight() < aktuell.getWeight()){
	        ergebnis.next();        
	      }
	      if (ergebnis.hasAccess()){
	        ergebnis.insert(aktuell);
	      } else {
	        ergebnis.append(aktuell);
	      }

	      pList.next();
	    }
	    return ergebnis;
	  }

	  public List<Vertex> gibZusammenhangskomponente(List<List<Vertex>> pZusammenhangskomponenten, Vertex pVertex){
	    List<Vertex> ergebnis = null;
	    pZusammenhangskomponenten.toFirst();
	    while (pZusammenhangskomponenten.hasAccess() && ergebnis == null){
	      List<Vertex> aktuelleKomponente = pZusammenhangskomponenten.getContent();
	      aktuelleKomponente.toFirst();
	      while (aktuelleKomponente.hasAccess() && ergebnis == null){
	        if (aktuelleKomponente.getContent() == pVertex){
	          ergebnis = aktuelleKomponente;
	        }
	        aktuelleKomponente.next();
	      }
	      pZusammenhangskomponenten.next();
	    }
	    return ergebnis;
	  } 
	  
  public Graph gibSpannbaumKruskal(Graph pGraph){
	    //Knotenliste abfragen.
	    List<Vertex> knotenListe = pGraph.getVertices();

	    //Kantenliste abfragen und aufsteigend sortieren.
	    List<Edge> kantenListe = this.gibAufsteigendSortiert(pGraph.getEdges());

	    //Ergebnisgraphen (Spannbaum) erstellen und Knoten uebertragen.
	    Graph spannbaum = new Graph();
	    knotenListe.toFirst();
	    while (knotenListe.hasAccess()){ 
	      spannbaum.addVertex(knotenListe.getContent());
	      knotenListe.next();
	    }

	    //Liste aller Zusammenhangskomponenten erstellen. Zu Beginn ist jeder Knoten 
	    //eine eigene Zusammenhangskomponenten in einer Liste.
	    List<List<Vertex>> zusammenhangskomponenten = new List<List<Vertex>>();
	    knotenListe.toFirst();
	    while (knotenListe.hasAccess()){
	      List<Vertex> komponente = new List<Vertex>();
	      komponente.append(knotenListe.getContent());
	      zusammenhangskomponenten.append(komponente);
	      knotenListe.next();
	    }

	    //Kanten in aufsteigender Sortierung einfuegen, sofern sie Zusammenhangskomponenten 
	    //verbinden.
	    kantenListe.toFirst();
	    while (!kantenListe.isEmpty()){
	      //Aktuelle Kante wird ausgelesen.
	      Edge aktuelleKante = kantenListe.getContent();

	      //Zusamenhangskomponenten der Knoten der aktuellen Kante werden ermittelt.
	      List<Vertex> ersteKomponente = this.gibZusammenhangskomponente(zusammenhangskomponenten, aktuelleKante.getVertices()[0]);
	      List<Vertex> zweiteKomponente = this.gibZusammenhangskomponente(zusammenhangskomponenten, aktuelleKante.getVertices()[1]);

	      //Kante dem Spannbaum hinzufuegen, wenn die Knoten in unterschiedlichen Zusamenhangskomponenten sind und diese
	      //Zusammenhangskomponenten zusammenlegen.
	      if (ersteKomponente != zweiteKomponente){
	        ersteKomponente.concat(zweiteKomponente);
	        spannbaum.addEdge(aktuelleKante);
	        aktuelleKante.setMark(true);;
	        Vertex[] endknoten = aktuelleKante.getVertices();
	        endknoten[0].setMark(true);
	        endknoten[1].setMark(true);
	        markierteKanten = markierteKanten+endknoten[0].getID()+endknoten[1].getID()+" ";
	      }      

	      //Akteulle Kante loeschen, da sie abgearbeitet ist.
	      kantenListe.remove();
	    }

	    return spannbaum;
	  }


  public double berechneBaumlaenge(List<Edge> liste) {
    double summe = 0.0;
    liste.toFirst();
    while (liste.hasAccess() ) {
      Edge kante = liste.getContent();
      summe = summe + kante.getWeight();
      liste.next();
    }
    return summe;
  }



  
   public void run(){
	  bspGraph.setAllVertexMarks(false);
	  bspGraph.setAllEdgeMarks(false);
	  Graph spannbaum = this.gibSpannbaumKruskal(bspGraph);
	  double baumlaenge = this.berechneBaumlaenge(spannbaum.getEdges());
	  bspGraph.setAllVertexMarks(false);
	  textfeld.setText("Spannbaum Kruskal-Algorithmus\n");
	  textfeld.setText(textfeld.getText()+"Reihenfolge gewählter Kanten\n"+markierteKanten+"\nGesamtlänge: "+baumlaenge);
	  
   }

  
}
